@@ -16,9 +16,9 @@
 import unittest
 
 import mock
-from neutron.callbacks import resources
 from neutron.db import provisioning_blocks
 from neutron.plugins.ml2 import driver_context
+from neutron_lib.callbacks import resources
 
 from networking_generic_switch import exceptions
 from networking_generic_switch import generic_switch_mech as gsm
@@ -33,6 +33,7 @@ class TestGenericSwitchDriver(unittest.TestCase):
         self.switch_mock = mock.Mock()
         self.switch_mock.config = {'device_type': 'bar', 'spam': 'ham',
                                    'ip': 'ip'}
+        self.switch_mock._get_physical_networks.return_value = []
         patcher = mock.patch(
             'networking_generic_switch.devices.device_manager',
             return_value=self.switch_mock)
@@ -45,11 +46,53 @@ class TestGenericSwitchDriver(unittest.TestCase):
         mock_context = mock.create_autospec(driver_context.NetworkContext)
         mock_context.current = {'id': 22,
                                 'provider:network_type': 'vlan',
-                                'provider:segmentation_id': 22}
+                                'provider:segmentation_id': 22,
+                                'provider:physical_network': 'physnet1'}
 
         driver.create_network_postcommit(mock_context)
         self.switch_mock.add_network.assert_called_once_with(22, 22)
 
+    def test_create_network_postcommit_with_physnet(self, m_list):
+        driver = gsm.GenericSwitchDriver()
+        driver.initialize()
+        mock_context = mock.create_autospec(driver_context.NetworkContext)
+        mock_context.current = {'id': 22,
+                                'provider:network_type': 'vlan',
+                                'provider:segmentation_id': 22,
+                                'provider:physical_network': 'physnet1'}
+        self.switch_mock._get_physical_networks.return_value = ['physnet1']
+
+        driver.create_network_postcommit(mock_context)
+        self.switch_mock.add_network.assert_called_once_with(22, 22)
+
+    def test_create_network_postcommit_with_multiple_physnets(self, m_list):
+        driver = gsm.GenericSwitchDriver()
+        driver.initialize()
+        mock_context = mock.create_autospec(driver_context.NetworkContext)
+        mock_context.current = {'id': 22,
+                                'provider:network_type': 'vlan',
+                                'provider:segmentation_id': 22,
+                                'provider:physical_network': 'physnet1'}
+        self.switch_mock._get_physical_networks.return_value = ['physnet1',
+                                                                'physnet2']
+
+        driver.create_network_postcommit(mock_context)
+        self.switch_mock.add_network.assert_called_once_with(22, 22)
+        self.assertEqual(self.switch_mock.add_network.call_count, 1)
+
+    def test_create_network_postcommit_with_different_physnet(self, m_list):
+        driver = gsm.GenericSwitchDriver()
+        driver.initialize()
+        mock_context = mock.create_autospec(driver_context.NetworkContext)
+        mock_context.current = {'id': 22,
+                                'provider:network_type': 'vlan',
+                                'provider:segmentation_id': 22,
+                                'provider:physical_network': 'physnet1'}
+        self.switch_mock._get_physical_networks.return_value = ['physnet2']
+
+        driver.create_network_postcommit(mock_context)
+        self.assertFalse(self.switch_mock.add_network.called)
+
     @mock.patch('networking_generic_switch.generic_switch_mech.LOG')
     def test_create_network_postcommit_failure(self, m_log, m_list):
         driver = gsm.GenericSwitchDriver()
@@ -58,7 +101,8 @@ class TestGenericSwitchDriver(unittest.TestCase):
         mock_context = mock.create_autospec(driver_context.NetworkContext)
         mock_context.current = {'id': 22,
                                 'provider:network_type': 'vlan',
-                                'provider:segmentation_id': 22}
+                                'provider:segmentation_id': 22,
+                                'provider:physical_network': 'physnet1'}
 
         driver.create_network_postcommit(mock_context)
         self.switch_mock.add_network.assert_called_once_with(22, 22)
@@ -71,10 +115,52 @@ class TestGenericSwitchDriver(unittest.TestCase):
         mock_context = mock.create_autospec(driver_context.NetworkContext)
         mock_context.current = {'id': 22,
                                 'provider:network_type': 'vlan',
-                                'provider:segmentation_id': 22}
+                                'provider:segmentation_id': 22,
+                                'provider:physical_network': 'physnet1'}
 
         driver.delete_network_postcommit(mock_context)
-        self.switch_mock.del_network.assert_called_once_with(22)
+        self.switch_mock.del_network.assert_called_once_with(22, 22)
+
+    def test_delete_network_postcommit_with_physnet(self, m_list):
+        driver = gsm.GenericSwitchDriver()
+        driver.initialize()
+        mock_context = mock.create_autospec(driver_context.NetworkContext)
+        mock_context.current = {'id': 22,
+                                'provider:network_type': 'vlan',
+                                'provider:segmentation_id': 22,
+                                'provider:physical_network': 'physnet1'}
+        self.switch_mock._get_physical_networks.return_value = ['physnet1']
+
+        driver.delete_network_postcommit(mock_context)
+        self.switch_mock.del_network.assert_called_once_with(22, 22)
+
+    def test_delete_network_postcommit_with_multiple_physnets(self, m_list):
+        driver = gsm.GenericSwitchDriver()
+        driver.initialize()
+        mock_context = mock.create_autospec(driver_context.NetworkContext)
+        mock_context.current = {'id': 22,
+                                'provider:network_type': 'vlan',
+                                'provider:segmentation_id': 22,
+                                'provider:physical_network': 'physnet1'}
+        self.switch_mock._get_physical_networks.return_value = ['physnet1',
+                                                                'physnet2']
+
+        driver.delete_network_postcommit(mock_context)
+        self.switch_mock.del_network.assert_called_once_with(22, 22)
+        self.assertEqual(self.switch_mock.del_network.call_count, 1)
+
+    def test_delete_network_postcommit_with_different_physnet(self, m_list):
+        driver = gsm.GenericSwitchDriver()
+        driver.initialize()
+        mock_context = mock.create_autospec(driver_context.NetworkContext)
+        mock_context.current = {'id': 22,
+                                'provider:network_type': 'vlan',
+                                'provider:segmentation_id': 22,
+                                'provider:physical_network': 'physnet1'}
+        self.switch_mock._get_physical_networks.return_value = ['physnet2']
+
+        driver.delete_network_postcommit(mock_context)
+        self.assertFalse(self.switch_mock.del_network.called)
 
     @mock.patch('networking_generic_switch.generic_switch_mech.LOG')
     def test_delete_network_postcommit_failure(self, m_log, m_list):
@@ -84,13 +170,34 @@ class TestGenericSwitchDriver(unittest.TestCase):
         mock_context = mock.create_autospec(driver_context.NetworkContext)
         mock_context.current = {'id': 22,
                                 'provider:network_type': 'vlan',
-                                'provider:segmentation_id': 22}
+                                'provider:segmentation_id': 22,
+                                'provider:physical_network': 'physnet1'}
 
         driver.delete_network_postcommit(mock_context)
-        self.switch_mock.del_network.assert_called_once_with(22)
+        self.switch_mock.del_network.assert_called_once_with(22, 22)
         self.assertEqual(1, m_log.error.call_count)
         self.assertIn('Failed to delete network', m_log.error.call_args[0][0])
 
+    @mock.patch('networking_generic_switch.generic_switch_mech.LOG')
+    def test_delete_network_postcommit_no_network_id(self, m_log, m_list):
+        driver = gsm.GenericSwitchDriver()
+        driver.initialize()
+        mock_context = mock.create_autospec(driver_context.NetworkContext)
+        mock_context.current = {'id': 22,
+                                'provider:network_type': 'vlan',
+                                'provider:segmentation_id': 22,
+                                'provider:physical_network': 'physnet1'}
+        self.switch_mock.del_network.side_effect = TypeError
+
+        driver.delete_network_postcommit(mock_context)
+        self.assertEqual([mock.call(22, 22), mock.call(22)],
+                         self.switch_mock.del_network.call_args_list)
+        self.assertEqual(m_log.warn.call_count, 1)
+
+        # Ensure the warning is only logged once.
+        driver.delete_network_postcommit(mock_context)
+        self.assertEqual(m_log.warn.call_count, 1)
+
     def test_delete_port_postcommit(self, m_list):
         driver = gsm.GenericSwitchDriver()
         driver.initialize()
@@ -168,6 +275,31 @@ class TestGenericSwitchDriver(unittest.TestCase):
         self.assertIsNone(driver.delete_port_postcommit(mock_context))
         self.switch_mock.delete_port.assert_not_called()
 
+    def test_delete_port_postcommit_no_segmentation_id(self, m_list):
+        driver = gsm.GenericSwitchDriver()
+        driver.initialize()
+        mock_context = mock.create_autospec(driver_context.PortContext)
+        mock_context.current = {'binding:profile':
+                                {'local_link_information':
+                                    [
+                                        {
+                                            'switch_info': 'foo',
+                                            'port_id': '2222'
+                                        }
+                                    ]
+                                 },
+                                'binding:vnic_type': 'baremetal',
+                                'binding:vif_type': 'other',
+                                'id': 'aaaa-bbbb-cccc'}
+        mock_context.network = mock.Mock()
+        mock_context.network.current = {'provider:segmentation_id': None,
+                                        'id': 'aaaa-bbbb-cccc'}
+        mock_context.segments_to_bind = [mock_context.network.current]
+
+        driver.delete_port_postcommit(mock_context)
+        self.switch_mock.delete_port.assert_called_once_with(
+            '2222', '1')
+
     @mock.patch.object(provisioning_blocks, 'provisioning_complete')
     def test_update_port_postcommit_not_bound(self, m_pc, m_list):
         driver = gsm.GenericSwitchDriver()
@@ -430,6 +562,9 @@ class TestGenericSwitchDriver(unittest.TestCase):
                                  },
                                 'binding:vnic_type': 'baremetal',
                                 'id': '123'}
+        mock_context.network.current = {
+            'provider:physical_network': 'physnet1'
+        }
         mock_context.segments_to_bind = [
             {
                 'segmentation_id': None,
@@ -446,6 +581,43 @@ class TestGenericSwitchDriver(unittest.TestCase):
                                       resources.PORT,
                                       'GENERICSWITCH')
 
+    @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
+    def test_bind_port_with_physnet(self, m_apc, m_list):
+        driver = gsm.GenericSwitchDriver()
+        driver.initialize()
+        mock_context = mock.create_autospec(driver_context.PortContext)
+        mock_context._plugin_context = mock.MagicMock()
+        mock_context.current = {'binding:profile':
+                                {'local_link_information':
+                                    [
+                                        {
+                                            'switch_info': 'foo',
+                                            'port_id': 2222
+                                        }
+                                    ]
+                                 },
+                                'binding:vnic_type': 'baremetal',
+                                'id': '123'}
+        mock_context.network.current = {
+            'provider:physical_network': 'physnet1'
+        }
+        mock_context.segments_to_bind = [
+            {
+                'segmentation_id': None,
+                'id': 123
+            }
+        ]
+        self.switch_mock._get_physical_networks.return_value = ['physnet1']
+
+        driver.bind_port(mock_context)
+        self.switch_mock.plug_port_to_network.assert_called_once_with(
+            2222, '1')
+        mock_context.set_binding.assert_called_with(123, 'other', {})
+        m_apc.assert_called_once_with(mock_context._plugin_context,
+                                      mock_context.current['id'],
+                                      resources.PORT,
+                                      'GENERICSWITCH')
+
     @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
     def test_bind_port_unknown_switch(self, m_apc, m_list):
         driver = gsm.GenericSwitchDriver()
@@ -463,6 +635,9 @@ class TestGenericSwitchDriver(unittest.TestCase):
                                  },
                                 'binding:vnic_type': 'baremetal',
                                 'id': '123'}
+        mock_context.network.current = {
+            'provider:physical_network': 'physnet1'
+        }
         mock_context.segments_to_bind = [
             {
                 'segmentation_id': None,
@@ -470,6 +645,39 @@ class TestGenericSwitchDriver(unittest.TestCase):
             }
         ]
         self.assertIsNone(driver.bind_port(mock_context))
+        self.assertFalse(mock_context.set_binding.called)
+        self.switch_mock.plug_port_to_network.assert_not_called()
+        self.assertFalse(m_apc.called)
+
+    @mock.patch.object(provisioning_blocks, 'add_provisioning_component')
+    def test_bind_port_with_different_physnet(self, m_apc, m_list):
+        driver = gsm.GenericSwitchDriver()
+        driver.initialize()
+        mock_context = mock.create_autospec(driver_context.PortContext)
+        mock_context._plugin_context = mock.MagicMock()
+        mock_context.current = {'binding:profile':
+                                {'local_link_information':
+                                    [
+                                        {
+                                            'switch_info': 'bar',
+                                            'port_id': 2222
+                                        }
+                                    ]
+                                 },
+                                'binding:vnic_type': 'baremetal',
+                                'id': '123'}
+        mock_context.network.current = {
+            'provider:physical_network': 'physnet1'
+        }
+        mock_context.segments_to_bind = [
+            {
+                'segmentation_id': None,
+                'id': 123
+            }
+        ]
+        self.switch_mock._get_physical_networks.return_value = ['physnet2']
+        self.assertIsNone(driver.bind_port(mock_context))
+        self.assertFalse(mock_context.set_binding.called)
         self.switch_mock.plug_port_to_network.assert_not_called()
         self.assertFalse(m_apc.called)
 
