@@ -5,7 +5,7 @@
 set -ex
 
 ZUUL_CLONER=/usr/zuul-env/bin/zuul-cloner
-BRANCH_NAME=master
+BRANCH_NAME=stable/queens
 
 CONSTRAINTS_FILE=$1
 shift
@@ -37,6 +37,9 @@ else
 fi
 
 # Install the rest of the requirements as normal
-$install_cmd -U $*
-
+if [ "$#" -gt 0 ]; then
+    $install_cmd -U $*
+else
+    $install_cmd -U .
+fi
 exit $?
